clear all
close all

%% DEFINITION OF PARAMETERS
% Loading data according to weekend or weekday simulation
load('data')
we = 0;
if (we == 0)
    measzone1 = measzone1_wd;
    measzone2 = measzone2_wd;
    predzone1 = predzone1_wd;
    predzone2 = predzone2_wd;
    sysp = sysp_wd;
    cons = cons_wd;
    begin_day = 66;
    Nd = 21;
else
    measzone1 = measzone1_we;
    measzone2 = measzone2_we;
    predzone1 = predzone1_we;
    predzone2 = predzone2_we;
    sysp = sysp_we;
    cons = cons_we;
    begin_day = 26;    
    Nd = 9;
end

clear cons_wd cons_we sysp_wd sysp_we ...
    measzone1_wd measzone1_we predzone1_wd predzone1_we...
    measzone2_wd measzone2_we predzone2_wd predzone2_we 

perf_pred = 0;
if perf_pred 
    predzone1 = measzone1;
    predzone2 = measzone2;
end

naive1 = 0;
naive2 = 0;


%% DETERMINATION OF DEMAND PROFILE
e = []; f = []; qm = [];

b = 2000;
if b == 1000
    e = 69;
elseif b == 1500
    e = 54;
else
    e = 46.5;
end

for i = 1:Nd+2
    day = (i-1) + begin_day;                    %day = (i-1) + 91;
    [pmi,qmi] = p_q_estimates(day,cons,sysp);
    f = [f (e-pmi)./qmi];
    pm = pmi;
    qm = [qm qmi];
end

e = e*f./f;
a = e./f;
b = 1./f;

%% PARAMETERS OF THE GENERATORS
% GENERATOR 1
%k_adj_1 = 5e-12; not needed
aw1 = 10;
a_1 = 10;
b_1 = 1.5;
c_1 = 0.0005;
share1 = 0.5;

if ~share1
    aw1 = 0;
end

% GENERATOR 2
%k_adj_2 = k_adj_1; not needed
aw2 = aw1;
a_2 = 10;
b_2 = 1.5;
c_2 = 0.0005;
share2 = share1;

%% SIMULATION
% Initial conditions
alpha_1 = b_1*ones(24,length(e(1,:)));        % the alphas are always equal to the constant term in the marginal cost (b)
alpha_2 = b_2*ones(24,length(e(1,:)));
beta_1 = [1*c_2*ones(24,1) zeros(24,length(e(1,:))-1)];
beta_2 = [1*c_2*ones(24,1) zeros(24,length(e(1,:))-1)];

q_1 = beta_1*0;
q_2 = q_1;
p = q_1;

qmMax = max(max(qm));

q_1w = wind_generator(share1,qmMax,Nd+1,1,begin_day,measzone1); %first value is for hour=1 of April 1st
q_2w = wind_generator(share2,qmMax,Nd+1,1,begin_day,measzone2); %first value is for hour=1 of April 1st
q_1wm = 0*q_1w;   % forecast matrices initialised to 0
q_2wm = q_1wm;

for i = 1:Nd+2
    q_1wm(:,i) = pred_generator(share1,qmMax,i,1,begin_day,predzone1,perf_pred); %first value is for hour=1 of April 1st
    q_2wm(:,i) = pred_generator(share2,qmMax,i,1,begin_day,predzone2,perf_pred); %first value is for hour=1 of April 1st
end

for i = 2:Nd+2
    for j = 1:24
        beta_1(j,i) =   (-c_1 * beta_2(j,i-1) ^ 2 * f(j,i) * q_2wm(j,i-1) + c_1 * f(j,i) * alpha_2(j,i-1) * beta_2(j,i-1) ...
                        - c_1 * beta_2(j,i-1) * f(j,i) ^ 2 * q_1wm(j,i) - c_1 * beta_2(j,i-1) ^ 2 * f(j,i) *...
                        q_1wm(j,i) + c_1 * beta_2(j,i-1) * e(j,i)* f(j,i) - c_1 * beta_2(j,i-1) * f(j,i) ^ 2 * q_2wm(j,i-1) - ...
                        2 * c_1 * beta_2(j,i-1) * f(j,i) * alpha_1(j,i) - 2 * beta_2(j,i-1) * f(j,i) ^ 2 * alpha_1(j,i) + ...
                        beta_2(j,i-1) ^ 2 * e(j,i)* f(j,i) - 2 * beta_2(j,i-1) ^ 2 * f(j,i) * alpha_1(j,i) - beta_2(j,i-1) ^ 2 ... 
                        * f(j,i) ^ 2 * q_2wm(j,i-1) + f(j,i) ^ 2 * alpha_2(j,i-1) * beta_2(j,i-1) - 1 * beta_2(j,i-1) ^ 2 * f(j,i) ^ 2 * ...
                        q_1wm(j,i) + b_1 * f(j,i) ^ 2 * beta_2(j,i-1) + b_1 * f(j,i) * beta_2(j,i-1) ^ 2 + c_1 * beta_2(j,i-1) ^ 2 * ...
                        e(j,i)- 1 * c_1 * alpha_1(j,i) * beta_2(j,i-1) ^ 2 + c_1 * f(j,i) ^ 2 * alpha_2(j,i-1) - 1 * c_1 * ...
                        f(j,i) ^ 2 * alpha_1(j,i)) / (-1 * beta_2(j,i-1) * f(j,i) ^ 2 * q_1wm(j,i) - 1 * beta_2(j,i-1) ^ 2 * ...
                        f(j,i) * q_1wm(j,i) + beta_2(j,i-1) * e(j,i)* f(j,i) - 1 * beta_2(j,i-1) * f(j,i) ^ 2 * q_2wm(j,i-1) - 1 * beta_2(j,i-1) ^ 2 ...
                        * f(j,i) * q_2wm(j,i-1) + f(j,i) * alpha_2(j,i-1) * beta_2(j,i-1) - 2 * b_1 * beta_2(j,i-1) * f(j,i) + beta_2(j,i-1) ^ 2 * e(j,i)+ ...
                        f(j,i) ^ 2 * alpha_2(j,i-1) - 1 * b_1 * f(j,i) ^ 2 - 1 * b_1 * beta_2(j,i-1) ^ 2);

        beta_2(j,i) =   (-c_2 * beta_1(j,i-1) ^ 2 * f(j,i) * q_1wm(j,i-1) + c_2 * f(j,i) * alpha_1(j,i-1) * beta_1(j,i-1) ...
                        - c_2 * beta_1(j,i-1) * f(j,i) ^ 2 * q_2wm(j,i) - c_2 * beta_1(j,i-1) ^ 2 * f(j,i) *...
                        q_2wm(j,i) + c_2 * beta_1(j,i-1) * e(j,i)* f(j,i) - c_2 * beta_1(j,i-1) * f(j,i) ^ 2 * q_1wm(j,i-1) - ...
                        2 * c_2 * beta_1(j,i-1) * f(j,i) * alpha_2(j,i) - 2 * beta_1(j,i-1) * f(j,i) ^ 2 * alpha_2(j,i) + ...
                        beta_1(j,i-1) ^ 2 * e(j,i)* f(j,i) - 2 * beta_1(j,i-1) ^ 2 * f(j,i) * alpha_2(j,i) - beta_1(j,i-1) ^ 2 ... 
                        * f(j,i) ^ 2 * q_1wm(j,i-1) + f(j,i) ^ 2 * alpha_1(j,i-1) * beta_1(j,i-1) - 1 * beta_1(j,i-1) ^ 2 * f(j,i) ^ 2 * ...
                        q_2wm(j,i) + b_2 * f(j,i) ^ 2 * beta_1(j,i-1) + b_2 * f(j,i) * beta_1(j,i-1) ^ 2 + c_2 * beta_1(j,i-1) ^ 2 * ...
                        e(j,i)- 1 * c_2 * alpha_2(j,i) * beta_1(j,i-1) ^ 2 + c_2 * f(j,i) ^ 2 * alpha_1(j,i-1) - 1 * c_2 * ...
                        f(j,i) ^ 2 * alpha_2(j,i)) / (-1 * beta_1(j,i-1) * f(j,i) ^ 2 * q_2wm(j,i) - 1 * beta_1(j,i-1) ^ 2 * ...
                        f(j,i) * q_2wm(j,i) + beta_1(j,i-1) * e(j,i)* f(j,i) - 1 * beta_1(j,i-1) * f(j,i) ^ 2 * q_1wm(j,i-1) - 1 * beta_1(j,i-1) ^ 2 ...
                        * f(j,i) * q_1wm(j,i-1) + f(j,i) * alpha_1(j,i-1) * beta_1(j,i-1) - 2 * b_2 * beta_1(j,i-1) * f(j,i) + beta_1(j,i-1) ^ 2 * e(j,i)+ ...
                        f(j,i) ^ 2 * alpha_1(j,i-1) - 1 * b_2 * f(j,i) ^ 2 - 1 * b_2 * beta_1(j,i-1) ^ 2);     
        if naive1
            beta_1(j,i) = c_1;
        end
        if naive2
            beta_2(j,i) = c_2;          
        end
    end
end

p = (-beta_1 .* beta_2 .* f.* q_1w + beta_1 .* beta_2 .* e+ beta_1 .* f.* alpha_2 - ...
    beta_1 .* beta_2 .* f.* q_2w + alpha_1 .* beta_2 .* f) ./ (beta_2 .* beta_1 + ...
    beta_2 .* f+ f.* beta_1);

q_1 = (p-alpha_1)./beta_1;
q_2 = (p-alpha_2)./beta_2; 


%% CUTTING AND RESHAPING
e = e(:,1:Nd+1);
f = f(:,1:Nd+1);
b = b(:,1:Nd+1);
beta_1 = beta_1(:,1:Nd+1);
beta_2 = beta_2(:,1:Nd+1);
alpha_1 = alpha_1(:,1:Nd+1);
alpha_2 = alpha_2(:,1:Nd+1);
p = p(:,1:Nd+1);
q_1 = q_1(:,1:Nd+1);
q_2 = q_2(:,1:Nd+1);
q_1w = q_1w(:,1:Nd+1);
q_2w = q_2w(:,1:Nd+1);
q_1wm = q_1wm(:,1:Nd+1);
q_2wm = q_2wm(:,1:Nd+1);

e = reshape(e,1,24*(Nd+1));
f = reshape(f,1,24*(Nd+1));
b = reshape(b,1,24*(Nd+1));
beta_1 = reshape(beta_1,1,24*(Nd+1));
beta_2 = reshape(beta_2,1,24*(Nd+1));
alpha_1 = reshape(alpha_1,1,24*(Nd+1));
alpha_2 = reshape(alpha_2,1,24*(Nd+1));
p = reshape(p,1,24*(Nd+1));
q_1 = reshape(q_1,1,24*(Nd+1));
q_2 = reshape(q_2,1,24*(Nd+1));
q_1w = reshape(q_1w,1,24*(Nd+1));
q_2w = reshape(q_2w,1,24*(Nd+1));
q_1wm = reshape(q_1wm,1,24*(Nd+1));
q_2wm = reshape(q_2wm,1,24*(Nd+1));

%% PERFORMANCE INDICES
q_1tot = q_1+q_1w;
q_2tot = q_2+q_2w;
profit_1 = p.*q_1tot-(aw1+a_1+b_1*q_1+0.5*c_1*q_1.^2);
profit_2 = p.*q_2tot-(aw2+a_2+b_2*q_2+0.5*c_2*q_2.^2);
soc_bft = e.*(q_1tot + q_2tot) - 0.5*f.*(q_1tot + q_2tot).^2 - (b_1*q_1 + 0.5*c_1*q_1.^2 + b_2*q_2 + 0.5*c_2*q_2.^2);

%% FIGURES AND RESULTS
% figure
% x = 0:100000;
% plot(x,mean(e)-mean(f)*x,'b')
% hold on
% plot(x,(mean(beta_1)*mean(beta_2)*x+mean(alpha_1)*mean(beta_2)+mean(alpha_2)*mean(beta_1))/(mean(beta_1)+mean(beta_2)),'r')
% legend('demand','combined supply')
% 
% figure
% plot(p)
% title('price')
% ylim([0 1.1*max(p)])

% figure
% hold on
% grid on
% xlabel('time [hours]')
% ylabel('Quantity [MWh]')
% subplot(2,1,1)
% title('generator 1')
% hold on
% plot(q_1tot,'b')
% plot(q_1,'r')
% plot(q_1w,'g')
% plot(q_1tot-circshift(q_1tot',24)','k')
% legend('q_1tot','q_1','q_1w','q_1tot(t)-q_1tot(t-1)')
% subplot(2,1,2)
% title('generator 2')
% hold on
% plot(q_2tot,'b')
% plot(q_2,'r')
% plot(q_2w,'g')
% plot(q_2tot-circshift(q_2tot',24)','k')
% legend('q_2tot','q_2','q_2w','q_2tot(t)-q_2tot(t-1)')
% 
% figure
% hold on
% plot(beta_1/c_1,'b')
% plot(beta_2/c_2,'r')
% title('betas')
% legend('beta1','beta2')

% figure
% hold on
% plot(profit_1,'b')
% plot(profit_2,'r')
% title('profits')
% legend('profit 1','profit 2')
% ylim([0 1.1*max([profit_1 profit_2])])
% 
% figure
% hold on
% subplot(2,1,1)
% plot(e,'b')
% title('e and f')
% legend('e')
% subplot(2,1,2)
% plot(f,'r')
% legend('f')

t_1 = 3*24+1;
t_2 = t_1+(Nd-6)*24;

%fprintf('\n')
fprintf('%f\n', mean(mean(a)))
fprintf('%f\n', mean(b(t_1:t_2)))
fprintf('%f\n', mean(e(t_1:t_2)))
fprintf('%f\n', mean(f(t_1:t_2)))
fprintf('%f\n', mean(beta_1(t_1:t_2)/c_1))
fprintf('%f\n', mean(beta_2(t_1:t_2)/c_2))
fprintf('%f\n', 100*mean(q_1w(t_1:t_2)./q_1tot(t_1:t_2)))               % mean percentage of wind gen.1
fprintf('%f\n', 100*mean(q_2w(t_1:t_2)./q_2tot(t_1:t_2)))               % mean percentage of wind gen.2
fprintf('%f\n', mean(q_1(t_1:t_2)))                                     % mean produced quantity gen.1
fprintf('%f\n', mean(q_2(t_1:t_2)))                                     % mean produced quantity gen.2   
fprintf('%f\n', mean(q_1tot(t_1:t_2)))                                  % mean total quantity gen.1
fprintf('%f\n', mean(q_2tot(t_1:t_2)))                                  % mean total quantity gen.2
fprintf('%f\n', 24*mean(profit_1(t_1:t_2)))                             % daily profit gen.1
fprintf('%f\n', 24*mean(profit_2(t_1:t_2)))                             % daily profit gen.2
fprintf('%f\n', mean(p(t_1:t_2)))                                       % mean price
fprintf('%f\n', 24*mean(soc_bft(t_1:t_2)))                              % daily social benefit
fprintf('\n')