%% DEFINITION OF PARAMETERS
clear all
close all

% Loading data according to weekend or weekday simulation
load('data')
we = 0;
if (we == 0)
    measzone1 = measzone1_wd;
    measzone2 = measzone2_wd;
    predzone1 = predzone1_wd;
    predzone2 = predzone2_wd;
    sysp = sysp_wd;
    cons = cons_wd;
    begin_day = 66;
    Nd = 21;
else
    measzone1 = measzone1_we;
    measzone2 = measzone2_we;
    predzone1 = predzone1_we;
    predzone2 = predzone2_we;
    sysp = sysp_we;
    cons = cons_we;
    begin_day = 26;    
    Nd = 9;
end

clear den fin swe nor cons_wd cons_we sysp_wd sysp_we ...
    measzone1_wd measzone1_we predzone1_wd predzone1_we...
    measzone2_wd measzone2_we predzone2_wd predzone2_we 

perf_pred = 0;
if perf_pred 
    predzone1 = measzone1;
    predzone2 = measzone2;
end

naive1 = 0; %set 1 to make gen.1 use the naive strategy
naive2 = 0; %set 1 to make gen.2 use the naive strategy

%% DETERMINATION OF THE DEMAND PROFILE
e = []; f = []; qm = [];

b0 = 1500;
if b0 == 1000
    e = 69;
elseif b0 == 1500
    e = 54;
else
    e = 46.5;
end

for i = 1:Nd+1
    day = (i-1) + begin_day;                    %day = (i-1) + 91;
    [pmi,qmi] = p_q_estimates(day,cons,sysp);
    f = [f (e-pmi)./qmi];
    pm = pmi;
    qm = [qm qmi];
end

e = e*f./f;
a = e./f;
b = 1./f;

%% PARAMETERS OF THE GENERATORS
if b0 == 1000
    beta = 500;
elseif b0 == 1500
    beta = 700;
else
    beta = 900;
end
% GENERATOR 1
beta_1 = beta;
aw1 = 10;
a_1 = 10;
b_1 =  1.5;
c_1 = 0.0001;
share1 = 0.5;

if ~share1
    aw1 = 0;
end

% GENERATOR 2
beta_2 = beta;
aw2 = aw1;
a_2 = a_1;
b_2 =  b_1;
c_2 = c_1;
share2 = share1;

%% SIMULATION
% INITIAL CONDITIONS
qmMax = max(max(qm));
q_1w = wind_generator(share1,qmMax,Nd,1,begin_day,measzone1); %first value is for hour=1 of April 1st
q_2w = wind_generator(share2,qmMax,Nd,1,begin_day,measzone2); %first value is for hour=1 of April 1st
q_1wm = 0*q_1w;   % forecast matrices initialised to 0
q_2wm = q_1wm;

for i = 1:Nd+1
    q_1wm(:,i) = pred_generator(share1,qmMax,i,1,begin_day,predzone1,perf_pred); %first value is for hour=1 of April 2nd
    q_2wm(:,i) = pred_generator(share2,qmMax,i,1,begin_day,predzone2,perf_pred); %first value is for hour=1 of April 2nd
end

q_1 = [0.45*qm(:,1)-q_1wm(:,1) zeros(24,Nd)];
q_2 = [0.45*qm(:,1)-q_2wm(:,1) zeros(24,Nd)];
dpi2 = q_1*0;

for i = 2:Nd+1
    for j = 1:24
        q_1(j,i) = (1-beta_1*(c_1+2*f(j,i)))*q_1(j,i-1) - beta_1*f(j,i)*q_2(j,i-1) - beta_1*f(j,i)*(2*q_1wm(j,i)+q_2wm(j,i-1)) + beta_1*(e(j,i)-b_1);
        q_2(j,i) = (1-beta_2*(c_2+2*f(j,i)))*q_2(j,i-1) - beta_2*f(j,i)*q_1(j,i-1) - beta_2*f(j,i)*(2*q_2wm(j,i)+q_1wm(j,i-1)) + beta_2*(e(j,i)-b_2);

        dpi2(j,i) = -(c_2+2*f(j,i))*q_2(j,i-1) - f(j,i)*q_1(j,i-1) - f(j,i)*(2*q_2wm(j,i)+q_1wm(j,i-1)) + e(j,i)-b_2; 
        
        if naive1
            q_1(j,i) = q_1(j,i-1)+q_1wm(j,i-1)-q_1wm(j,i);   
        end
        if naive2
            q_2(j,i) = q_2(j,i-1)+q_2wm(j,i-1)-q_2wm(j,i);
        end
    end
end

%% reshape
q_1 = reshape(q_1(:,1:end),1,24*(Nd+1));
q_2 = reshape(q_2(:,1:end),1,24*(Nd+1));
q_1w = reshape(q_1w(:,1:Nd+1),1,24*(Nd+1));
q_2w = reshape(q_2w(:,1:Nd+1),1,24*(Nd+1));
q_1wm = reshape(q_1wm(:,1:Nd+1),1,24*(Nd+1));
q_2wm = reshape(q_2wm(:,1:Nd+1),1,24*(Nd+1));
e = reshape(e(:,1:end),1,24*(Nd+1));
f = reshape(f(:,1:end),1,24*(Nd+1));
a = reshape(a(:,1:end),1,24*(Nd+1));
b = reshape(b(:,1:end),1,24*(Nd+1));

%% indexes

q_1tot = q_1 + q_1wm; 
q_2tot = q_2 + q_2wm;
q_1 = q_1tot - q_1w;
q_2 = q_2tot - q_2w;

% q_1tot = q_1 + q_1w; 
% q_2tot = q_2 + q_2w;

p = e - f.*(q_1tot + q_2tot);
profit_1 = p.*q_1tot - (aw1 + a_1 + b_1*q_1 + 0.5*c_1*q_1.^2);
profit_2 = p.*q_2tot - (aw2 + a_2 + b_2*q_2 + 0.5*c_2*q_2.^2);
soc_bft = e.*(q_1tot + q_2tot) - 0.5*f.*(q_1tot + q_2tot).^2 - (b_1*q_1 + 0.5*c_1*q_1.^2 + b_2*q_2 + 0.5*c_2*q_2.^2);


t_1 = 3*24;%+1;
t_2 = t_1+(Nd-6)*24;

%% figures and results
fprintf('%f\n', mean(a(t_1:t_2)))
fprintf('%f\n', mean(b(t_1:t_2)))
fprintf('%f\n', 100*mean(q_1w(t_1:t_2)./q_1tot(t_1:t_2)))               % mean percentage of wind gen.1
fprintf('%f\n', 100*mean(q_2w(t_1:t_2)./q_2tot(t_1:t_2)))               % mean percentage of wind gen.2
fprintf('%f\n', mean(q_1(t_1:t_2)))                                     % mean produced quantity gen.1
fprintf('%f\n', mean(q_2(t_1:t_2)))                                     % mean produced quantity gen.2   
fprintf('%f\n', mean(q_1tot(t_1:t_2)))                                  % mean total quantity gen.1
fprintf('%f\n', mean(q_2tot(t_1:t_2)))                                  % mean total quantity gen.2
fprintf('%f\n', 24*mean(profit_1(t_1:t_2)))                             % daily profit gen.1
fprintf('%f\n', 24*mean(profit_2(t_1:t_2)))                             % daily profit gen.2
fprintf('%f\n', mean(p(t_1:t_2)))                                       % mean price
fprintf('%f\n', 24*mean(soc_bft(t_1:t_2)))                              % daily social benefit
fprintf('\n')


str_tit = strcat('b~', int2str(mean(b)));

figure
title(str_tit)
hold on
grid on
plot(p,'b')
xlabel('time [hours]')
ylabel('price [EUR]')

figure
title(str_tit)
hold on
grid on
xlabel('time [hours]')
ylabel('Quantity [MWh]')
subplot(2,1,1)
title('generator 1')
hold on
plot(q_1tot,'b')
plot(q_1,'r')
plot(q_1w,'g')
plot(q_1tot-circshift(q_1tot',24)','k')
legend('q_1tot','q_1','q_1w','q_1tot(t)-q_1tot(t-1)')
subplot(2,1,2)
title('generator 2')
hold on
plot(q_2tot,'b')
plot(q_2,'r')
plot(q_2w,'g')
plot(q_2tot-circshift(q_2tot',24)','k')
legend('q_2tot','q_2','q_2w','q_2tot(t)-q_2tot(t-1)')

figure
subplot(2,1,1)
hold on
plot(q_1wm,'b')
plot(q_1w,'r')
legend('q_1wm','q_1w')
subplot(2,1,2)
hold on
plot(q_2wm,'b')
plot(q_2w,'r')
legend('q_2wm','q_2w')